@@ -197,9 +197,10 @@ cvs_commit_time_close(const cvstime_t a, const cvstime_t b)
     return false;
 }
 
-static bool
-cvs_commit_match(const cvs_commit *a, const cvs_commit *b)
-/* are two CVS commits eligible to be coalesced into a changeset? */
+typedef enum {no, yes, maybe} tribool;
+
+static tribool
+cvs_commitid_match(const cvs_commit *a, const cvs_commit *b)
 {
     if (trust_commitids)
     {
@@ -208,10 +209,29 @@ cvs_commit_match(const cvs_commit *a, const cvs_commit *b)
 	 * each commit to track patch sets. Use it if present
 	 */
 	if (a->commitid && b->commitid)
-	    return a->commitid == b->commitid;
+	    return (a->commitid == b->commitid ? yes : no);
 	if (a->commitid || b->commitid)
-	    return false;
+	    return no;
     }
+
+    return maybe;
+}
+
+static bool
+cvs_commit_match(const cvs_commit *a, const cvs_commit *b)
+/* are two CVS commits eligible to be coalesced into a changeset? */
+{
+    tribool idcheck = cvs_commitid_match(a, b);
+
+    switch (idcheck) {
+    case yes:
+	return true;
+    case no:
+	return false;
+    case maybe:
+	break;
+    }
+
     if (!cvs_commit_time_close(a->date, b->date))
 	return false;
     if (a->log != b->log)
@@ -517,9 +537,11 @@ merge_branches(rev_ref **branches, int nbranch,
      * We want to keep the revisions array in the same order as branches
      * was passed to us (modulo removing items) if not the fast output
      * phase suffers badly.
+     *
      * However, for the purpose of computing cliques it is very useful to
      * have the array sorted by date (with special handling for null and
-     & tailed commits)
+     * tailed commits)
+     *
      * So, we will maintain a sort_buf which has indexes into the revisons
      * array but is sorted to help us find cliques. On any given iteration
      * sort_buf indexes point to the following types of commit
@@ -652,12 +674,17 @@ merge_branches(rev_ref **branches, int nbranch,
 	    if (c->tailed)
 		break;
 
-	    if (c != latest && can_match && !cvs_commit_time_close(latest->date, c->date)) {
+	    if (c != latest && can_match && !(cvs_commit_time_close(latest->date, c->date) || cvs_commitid_match(latest, c))) {
 		/*
 		 * Because we are in date order, once we hit something too
                  * far off, we can't get anything else in the clique -
                  * unless there are cases where things with the same commitid
                  * have wildly differing dates.
+		 *
+		 * But: the point of the cvs_commitid_match() call
+		 * above is to let us keep accumulating when we see a
+		 * matching commit id even if the commit is outside
+		 * the time window.
                  */
 		can_match = false;
 		/* how much of the array might now be unsorted */
